@@ -5,7 +5,13 @@ implementation of load cli command
 import sys
 import gzip
 import json
+import requests
 from datetime import datetime
+import re
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
 
 from ckanapi.errors import (NotFound, NotAuthorized, ValidationError,
     SearchIndexError)
@@ -21,7 +27,7 @@ except NameError:
 def load_things(ckan, thing, arguments,
         worker_pool=None, stdin=None, stdout=None, stderr=None):
     """
-    create and update datasets, groups and orgs
+    create and update datasets, groups, orgs and users
 
     The parent process creates a pool of worker processes and hands
     out json lines to each worker as they finish a task. Status of
@@ -33,7 +39,7 @@ def load_things(ckan, thing, arguments,
     if stdin is None:
         stdin = getattr(sys.stdin, 'buffer', sys.stdin)
     if stdout is None:
-        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
+        stdout = getattr(sys.__stdout__, 'buffer', sys.__stdout__)
     if stderr is None:
         stderr = getattr(sys.stderr, 'buffer', sys.stderr)
 
@@ -75,6 +81,9 @@ def load_things(ckan, thing, arguments,
 
     with quiet_int_pipe() as errors:
         for job_ids, finished, result in pool:
+            if not result:
+                # child exited with traceback
+                return 1
             timestamp, action, error, response = json.loads(
                 result.decode('utf-8'))
 
@@ -112,8 +121,17 @@ def load_things_worker(ckan, thing, arguments,
     """
     if stdin is None:
         stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+        # hack so that pdb can be used in extension/ckan
+        # code called by this worker
+        try:
+            sys.stdin = open('/dev/tty', 'rb')
+        except IOError:
+            pass
     if stdout is None:
-        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
+        stdout = getattr(sys.__stdout__, 'buffer', sys.__stdout__)
+        # hack so that "print debugging" can work in extension/ckan
+        # code called by this worker
+        sys.stdout = sys.stderr
 
     thing_show, thing_create, thing_update = {
         'datasets': (
@@ -122,6 +140,10 @@ def load_things_worker(ckan, thing, arguments,
             'group_show', 'group_create', 'group_update'),
         'organizations': (
             'organization_show', 'organization_create', 'organization_update'),
+        'users': (
+            'user_show', 'user_create', 'user_update'),
+        'related':(
+            'related_show','related_create','related_update'),
         }[thing]
 
     def reply(action, error, response):
@@ -143,7 +165,7 @@ def load_things_worker(ckan, thing, arguments,
             reply('read', 'UnicodeDecodeError', unicode(e))
             continue
 
-        if obj:
+        if obj is not None:
             existing = None
             if not arguments['--create-only']:
                 # use either id or name to locate existing records
@@ -151,7 +173,10 @@ def load_things_worker(ckan, thing, arguments,
                 if name:
                     try:
                         existing = ckan.call_action(thing_show,
-                            {'id': name, 'include_datasets': False})
+                            {'id': name,
+                             'include_datasets': False,
+                             'include_password_hash': True,
+                            })
                     except NotFound:
                         pass
                     except NotAuthorized as e:
@@ -182,6 +207,15 @@ def load_things_worker(ckan, thing, arguments,
                     r = ckan.call_action(thing_update, obj)
                 else:
                     r = ckan.call_action(thing_create, obj)
+                if thing == 'datasets' and 'resources' in obj:# check if it is needed to upload resources when creating/updating packages
+                    _upload_resources(ckan,obj,arguments)
+                elif thing in ['groups','organizations'] and 'image_display_url' in obj:   #load images for groups and organizations
+                    if arguments['--upload-logo']:
+                        users = obj['users']
+                        _upload_logo(ckan,obj)
+                        obj.pop('image_upload')
+                        obj['users'] = users
+                        ckan.call_action(thing_update,obj)
             except ValidationError as e:
                 reply(act, 'ValidationError', e.error_dict)
             except SearchIndexError as e:
@@ -191,8 +225,7 @@ def load_things_worker(ckan, thing, arguments,
             except NotFound:
                 reply(act, 'NotFound', obj)
             else:
-                reply(act, None, r['name'])
-
+                reply(act, None, r.get('name',r.get('id')))
 
 def _worker_command_line(thing, arguments):
     """
@@ -213,6 +246,8 @@ def _worker_command_line(thing, arguments):
         + a('--apikey')
         + b('--create-only')
         + b('--update-only')
+        + b('--upload-resources')
+        + b('--upload-logo')
         )
 
 
@@ -231,3 +266,33 @@ def _copy_from_existing_for_update(obj, existing, thing):
     if thing in ('organizations', 'groups'):
         if 'users' not in obj and 'users' in existing:
             obj['users'] = existing['users']
+
+def _upload_resources(ckan,obj,arguments):
+    resources = obj['resources']
+    if not arguments['--upload-resources']:
+        return
+    for resource in resources:
+        if resource.get('url_type') != 'upload':
+            continue
+
+        f = requests.get(resource['url'],stream=True)
+        name = resource['url'].rsplit('/',1)[-1]
+        ckan.call_action('resource_patch',
+            {'id':resource['id']},
+            files={'upload':(name, f.raw)})
+
+
+def _upload_logo(ckan,obj):
+    for key in obj.keys():
+        if isinstance(obj[key],(dict,list)):
+            obj.pop(key)                            #dict/list objects can't be encoded
+    if urlparse(obj['image_url']).netloc:                  # logo is an external link
+        obj['clear_upload'] = True
+        obj['image_upload'] = obj['image_url']
+    else:
+        f = requests.get(obj['image_display_url'],stream=True)
+        name,ext = obj['image_url'].rsplit('.',1)  #reformulate image_url for new site
+        new_name = re.sub('[0-9\.-]','',name)
+        new_url = new_name+'.'+ext
+        obj['image_upload'] = (new_url, f.raw)
+    ckan.action.group_update(**obj)
